%code requires {

  #define YYERROR_VERBOSE
  #include <iostream>
  #include "instructions.h"
  class LoopLexer; // forward declaration

} // Yacc Deklarationen

%defines
%define parser_class_name {LoopParser}
%parse-param { LoopLexer &lexer }
%parse-param { LoopProgram** result }
%lex-param { LoopLexer &lexer }

%union
{
    std::string* stringval;
    int* intval;
    LoopInstruction* instrval;
    LoopProgram* progval;
};

%skeleton "lalr1.cc"
%start S

%token SEMICOLON LOOP DO DONE ASSIGN ADD SUB
%token <intval> NUM
%token <stringval> ID

%left ADD SUB

%code {
    static int yylex(yy::LoopParser::semantic_type* yylval, LoopLexer &scanner);
}

%% // Produktionen
S
    : program
    {
        *result = $<progval>1;
    }
    ;

program
    :
    {
        $<progval>$ = new LoopProgram();
    }

    | program instruction
    {
        $<progval>$ = $<progval>1;
        $<progval>$->Instructions.push_back($<instrval>2);
    }
    ;

instruction
    : ID ASSIGN ID ADD NUM SEMICOLON
    {
        $<instrval>$ = new Assignment( *$<stringval>1, *$<stringval>3, *$<intval>5 );
        delete $<stringval>1;
        delete $<intval>5;
        delete $<stringval>3;
    }

    | ID ASSIGN ID SUB NUM SEMICOLON
    {
        $<instrval>$ = new Assignment( *$<stringval>1, *$<stringval>3, (-1) * (*$<intval>5) );
        delete $<stringval>1;
        delete $<intval>5;
        delete $<stringval>3;
    }

    | LOOP ID DO program DONE
    {
        $<instrval>$ = new LoopVarLoop( *$<stringval>2, $<progval>4 );
        delete $<stringval>2;
    }

    | LOOP NUM DO program DONE
    {
        $<instrval>$ = new LoopConstLoop( *$<intval>2, $<progval>4 );
        delete $<intval>4;
    }
    ;

%%

#include "looplexer.h"

void yy::LoopParser::error(const std::string& s)
{
    std::cerr << "SYNTACTIC ERROR (Line " << lexer.GetCurrentLineNumber() << "): " << s << std::endl;
}

static int yylex(yy::LoopParser::semantic_type* yylval, LoopLexer &lexer)
{
    return lexer.yylex(yylval);
}
